import { buildSync } from "esbuild";

const build = () => {
  const srcDir = "src/";
  const outputDir = "build/";

  /**
   * @param {Record<string,string>} acc
   * @param {string} cur
   */
  function arrToObj(acc, cur) {
    const path = cur.split("/");
    const out = path[path.length - 1].replace(".ts", "");
    return {
      ...acc,
      [out]: cur,
    };
  }

  /** @type {Partial<import("esbuild").BuildOptions>} */
  const baseOptions = {
    bundle: true,
    minify: true,
    target: "es2020",
    sourcemap: false,
    outdir: outputDir,
    format: "esm",
    define: {
      global: "window",
      "process.env.NODE_ENV": '"production"',
    },
  };

  // Codemirror
  buildSync({
    ...baseOptions,
    outdir: outputDir,
    entryPoints: [srcDir + "codemirror-core.ts"],
  });

  // Codemirror-ts worker
  buildSync({
    ...baseOptions,
    entryPoints: [srcDir + "codemirror-ts.worker.ts"],
    outdir: outputDir,
    format: "iife",
    globalName: "CodemirrorTsWorker",
  });

  buildSync({
    ...baseOptions,
    outdir: outputDir,
    ignoreAnnotations: true, // required for codemirror-emacs
    entryPoints: [
      "codemirror-ts.ts",
      "codemirror-vim.ts",
      "codemirror-emacs.ts",
      "codemirror-emmet.ts",
      "codemirror-codeium.ts",
      "codemirror-line-numbers-relative.ts",
      "languages/codemirror-lang-json.ts",
      "languages/codemirror-lang-markdown.ts",
      "languages/codemirror-lang-python.ts",
      "languages/codemirror-lang-scss.ts",
      "languages/codemirror-lang-coffeescript.ts",
      "languages/codemirror-lang-livescript.ts",
      "languages/codemirror-lang-php.ts",
      "languages/codemirror-lang-cpp.ts",
      "languages/codemirror-lang-sql.ts",
      "languages/codemirror-lang-wast.ts",
      "languages/codemirror-lang-ruby.ts",
      "languages/codemirror-lang-go.ts",
      "languages/codemirror-lang-perl.ts",
      "languages/codemirror-lang-lua.ts",
      "languages/codemirror-lang-r.ts",
      "languages/codemirror-lang-julia.ts",
      "languages/codemirror-lang-scheme.ts",
      "languages/codemirror-lang-clojure.ts",
      "languages/codemirror-lang-tcl.ts",
      "languages/codemirror-lang-less.ts",
      "languages/codemirror-lang-stylus.ts",
      "languages/codemirror-lang-vue.ts",
      "languages/codemirror-lang-rust.ts",
      "languages/codemirror-lang-swift.ts",
    ]
      .map((x) => srcDir + x)
      .reduce(arrToObj, {}),
    external: ["@codemirror/*", "@lezer/*"],
  });
};

build();
